"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelFactory = exports.GraphQLHelper = void 0;
const graphql_parse_resolve_info_1 = require("graphql-parse-resolve-info");
const lodash_1 = __importDefault(require("lodash"));
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const config_base_1 = require("../config-base");
const FIELDS_SELECTION_FLAG = config_base_1.ConfigBase.get('FIELDS_SELECTION_FLAG', (0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkBoolean, false));
const QUERY_OPERATION_START_WITH = config_base_1.ConfigBase.get('QUERY_OPERATION_START_WITH', (x) => {
    x = x ? JSON.parse(x) : ['browse', 'read', 'onBoardBrowse', 'onBoardRead'];
    if (!Array.isArray(x))
        throw new Error('invalid Array');
    return x;
});
const QUERY_OPERATION_BROWSE_START_WITH = config_base_1.ConfigBase.get('QUERY_OPERATION_BROWSE_START_WITH', (x) => {
    x = x ? JSON.parse(x) : ['browse', 'onBoardBrowse'];
    if (!Array.isArray(x))
        throw new Error('invalid Array');
    return x;
});
class GraphQLHelper {
    static #mapping;
    static setMap(jsonObject) {
        this.#mapping = jsonObject;
    }
    static fieldsSelection(info) {
        const parsedResolveInfoFragment = (0, graphql_parse_resolve_info_1.parseResolveInfo)(info);
        const { fields } = (0, graphql_parse_resolve_info_1.simplifyParsedResolveInfoFragmentWithType)(parsedResolveInfoFragment, info.returnType);
        /// need to stay as undefined if it has nothing
        let selection;
        for (const [key, value] of Object.entries(fields)) {
            if (lodash_1.default.isEqual(value['fieldsByTypeName'], {})) {
                selection ??= {};
                selection[key] = true;
            }
        }
        return selection;
    }
    static itemsFieldsSelection(info, entity) {
        const parsedResolveInfoFragment = (0, graphql_parse_resolve_info_1.parseResolveInfo)(info);
        const { fields } = (0, graphql_parse_resolve_info_1.simplifyParsedResolveInfoFragmentWithType)(parsedResolveInfoFragment, info.returnType);
        /// need to stay as undefined if it has nothing
        let selection;
        for (const [key, value] of Object.entries(fields)) {
            if (lodash_1.default.isEqual(value['fieldsByTypeName'], {})) {
                selection ??= {};
                selection[key] = true;
            }
            else {
                if (value['fieldsByTypeName'][entity]) {
                    const found = value['fieldsByTypeName'][entity];
                    selection = this.#get(found);
                }
            }
        }
        return selection;
    }
    static requestQueryDSLAtResolverLevel(info) {
        const parsedResolveInfoFragment = (0, graphql_parse_resolve_info_1.parseResolveInfo)(info);
        const { fields } = (0, graphql_parse_resolve_info_1.simplifyParsedResolveInfoFragmentWithType)(parsedResolveInfoFragment, info.returnType);
        /// need to stay as undefined if it has nothing
        let selection;
        for (const [key, value] of Object.entries(fields)) {
            if (lodash_1.default.isEqual(value['fieldsByTypeName'], {})) {
                selection ??= {};
                selection[key] = true;
            }
            else {
                const found = value['fieldsByTypeName'][this.#mapping[key]];
                selection[key] = this.#resolvingKeys(found);
            }
        }
        return selection;
    }
    static #get(fields) {
        const selection = {};
        for (const [key, value] of Object.entries(fields)) {
            if (lodash_1.default.isEqual(value['fieldsByTypeName'], {}))
                selection[key] = true;
        }
        return selection;
    }
    static #resolvingKeys(fields) {
        const selection = {};
        for (const [key, value] of Object.entries(fields)) {
            if (lodash_1.default.isEqual(value['fieldsByTypeName'], {}))
                selection[key] = true;
            else {
                selection[key] = this.#resolvingKeys(value['fieldsByTypeName'][this.#mapping[key]]);
            }
        }
        return selection;
    }
    static queryDetector(baseString, stringToFind) {
        if (baseString) {
            const query = baseString.replace(/(?:\\[rn]|[\r\n]+)+/g, '');
            const regex = RegExp(`${stringToFind}\\W?(\\s*?){(.*?)}`, 'gms');
            let result = query.match(regex);
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `queryDetector[${stringToFind}]: ${JSON.stringify(result)}`);
            if (result && result.length > 0)
                result = result[0].match(/[^{\\}]+(?=})/g);
            if (result && result.length > 0)
                return result[0];
        }
        return '';
    }
    static getResolvers(obj) {
        const resolvers = [];
        const keys = Object.keys(obj);
        for (let i = 0, l = keys.length; i < l; ++i) {
            if (obj[keys[i]]['Query'] || obj[keys[i]]['Mutation']) {
                resolvers.push(obj[keys[i]]);
            }
        }
        return resolvers;
    }
}
exports.GraphQLHelper = GraphQLHelper;
class ModelFactory {
    listOfApis;
    type;
    listToResolve;
    query = {};
    mutation = {};
    constructor(listOfApis, type, listToResolve = []) {
        this.listOfApis = listOfApis;
        this.type = type;
        this.listToResolve = listToResolve;
        if (listOfApis)
            this.#processOperation(this.listOfApis);
    }
    resolve(fn, args, context, resolveInfo, isBrowse = false) {
        if (FIELDS_SELECTION_FLAG) {
            if (isBrowse)
                args.select = GraphQLHelper.itemsFieldsSelection(resolveInfo, this.type);
            else
                args.select = GraphQLHelper.fieldsSelection(resolveInfo);
            this.#selectOrInclude(args, context, true);
        }
        else {
            this.#selectOrInclude(args, context);
        }
        return fn(args, context.accessInfo);
    }
    #processOperation(allComponents) {
        const functions = Object.values(allComponents);
        functions.forEach((component) => {
            if (component.constructor.name === 'AsyncFunction') {
                if (this.#checkName(QUERY_OPERATION_START_WITH, component.name)) {
                    this.query[component.name] = (_data, args, context, resolveInfo) => this.resolve(component, args, context, resolveInfo, this.#checkName(QUERY_OPERATION_BROWSE_START_WITH, component.name));
                }
                else {
                    this.mutation[component.name] = (_data, args, context, resolveInfo) => this.resolve(component, args, context, resolveInfo);
                }
            }
        });
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `${this.type} has ${functions.length} functions loaded`);
    }
    #checkName(listOfQueryName, functionName) {
        for (const iterator of listOfQueryName) {
            if (functionName.startsWith(iterator)) {
                return true;
            }
        }
        return false;
    }
    #selectOrInclude(args, context, isSelect = false) {
        const key = isSelect ? 'select' : 'include';
        for (const iterator of this.listToResolve) {
            const result = GraphQLHelper.queryDetector(JSON.stringify(context?.req?.body?.query), iterator);
            if (!lodash_1.default.isEmpty(lodash_1.default.trim(result))) {
                args[key] ??= {};
                args[key] = Object.assign(args[key], { [iterator]: true, _count: true });
            }
        }
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.INFO, () => `Application Fields [${key}]: ${JSON.stringify(args[key])}`);
    }
}
exports.ModelFactory = ModelFactory;
//# sourceMappingURL=graphql.js.map